#ifndef CDC_SDK_API_H__
#define CDC_SDK_API_H__

typedef void*                   cdc_device;
typedef unsigned char           uint8_t;
typedef unsigned short          uint16_t;
typedef unsigned int            uint32_t;
typedef unsigned long long      uint64_t;

typedef signed char             int8_t;
typedef short                   int16_t;
typedef int                     int32_t;
typedef long long               int64_t;

#define ERROR_SUCCESS              0
#define ERROR_BAD_ARGUMENT         -1001
#define ERROR_GET_DEVICE_LIST_FAIL -1002
#define ERROR_SET_GPIO_DIR_FAIL    -1003
#define ERROR_SET_GPIO_FAIL        -1004
#define ERROR_GET_GPIO_FAIL        -1005

enum CHANNEL_SELECT {
    CHANNEL_SELECT_CH0 = 0,
    CHANNEL_SELECT_CH1,
    CHANNEL_SELECT_CH2,
    CHANNEL_SELECT_CH3
};

enum GPIO_PORT_NUMBER {
    GPIO_PORT_NUMBER_0 = 0,
    GPIO_PORT_NUMBER_1,
    GPIO_PORT_NUMBER_2,
    GPIO_PORT_NUMBER_3,
    GPIO_PORT_NUMBER_4,
    GPIO_PORT_NUMBER_5,
    GPIO_PORT_NUMBER_6,
    GPIO_PORT_NUMBER_7,
    GPIO_PORT_NUMBER_8,
    GPIO_PORT_NUMBER_9,
    GPIO_PORT_NUMBER_10
};

enum GPIO_VALUE {
    GPIO_VALUE_LOW = 0,
    GPIO_VALUE_HIGH,
};

enum GPIO_DIR {
    GPIO_DIR_INPUT = 0,
    GPIO_DIR_OUTPUT
};

struct cdc_usb_info {
    uint16_t vendor;
    uint16_t product;
    uint8_t *productString;
    uint16_t bcdDevice;
    uint32_t locationID;
};

#ifdef __cplusplus
extern "C" {
#endif
// Device
int32_t cdc_get_device_list_vid(uint16_t vid, cdc_device **list);
    
int32_t cdc_get_device_list_vid_pid(uint16_t vid, uint16_t pid, cdc_device **list);

void cdc_free_device_list(cdc_device *list);

int32_t cdc_get_usb_info(cdc_device *device, struct cdc_usb_info *info);

// GPIO
int32_t cdc_set_gpio_dir(cdc_device *device, enum CHANNEL_SELECT channel_select, enum GPIO_PORT_NUMBER port_number, enum GPIO_DIR dir);

int32_t cdc_write_gpio(cdc_device *device, enum CHANNEL_SELECT channel_select, enum GPIO_PORT_NUMBER port_number, enum GPIO_VALUE gpio_value);

int32_t cdc_read_gpio(cdc_device *device, enum CHANNEL_SELECT channel_select, enum GPIO_PORT_NUMBER port_number, uint8_t *result);

// Other
const char* cdc_get_version(void);
#ifdef __cplusplus
}
#endif

#endif
